<?php

namespace Lc\ShopBundle\Manager;

use Doctrine\ORM\EntityManager as DoctrineEntityManager;
use Doctrine\ORM\EntityManagerInterface;
use Lc\ShopBundle\Event\EntityManager\EntityManagerEvent;
//use Lc\ShopBundle\IModel\EntityInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * class EntityManager.
 *
 * @author Simon Vieille <simon@deblan.fr>
 */
class EntityManager
{
        protected $eventDispatcher;

        protected $entityManager;

        public function __construct(EventDispatcherInterface $eventDispatcher, EntityManagerInterface $entityManager)
        {
                $this->eventDispatcher = $eventDispatcher;
                $this->entityManager = $entityManager;
        }

        public function getRepository($className)
        {
                return $this->entityManager->getRepository($this->getEntityName($className));
        }


        public function create($entity): self
        {
                $this->persist($entity);
                $this->eventDispatcher->dispatch(new EntityManagerEvent($entity), EntityManagerEvent::CREATE_EVENT);

                return $this;
        }

        public function update($entity): self
        {
                $this->persist($entity);
                $this->eventDispatcher->dispatch(new EntityManagerEvent($entity), EntityManagerEvent::UPDATE_EVENT);

                return $this;
        }

        public function delete($entity): self
        {
                $this->remove($entity);
                $this->eventDispatcher->dispatch(new EntityManagerEvent($entity), EntityManagerEvent::DELETE_EVENT);

                return $this;
        }

        public function flush(): self
        {
                $this->entityManager->flush();

                return $this;
        }

        public function clear(): self
        {
                $this->entityManager->clear();

                return $this;
        }

        protected function persist($entity)
        {
                $this->entityManager->persist($entity);
        }

        public function getEntityName($className)
        {
                if (substr($className, -9) === 'Interface') {
                        return $this->entityManager->getClassMetadata($className)->getName();
                }else{
                        return $className;
                }

        }
}
